use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Alders
Alders'
Client
Comment
Dave
Eilam
Eilam's
Entity
Florian
Greg
Inc
Label
MAXMIND
MAXMIND's
MaxMind
MaxMind's
Me
Oschwald
Oschwald's
PayPal
Person
PivotalTracker
Project
ProjectIteration
PropertyAttributes
Ragwitz
Rolsky
Rolsky's
Story
Types
WebService
autarch
distro
drolsky
goschwald
lib
rafl
